# -*- coding: utf-8 -*-

# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
# Copyright (C) 2010-2011 by Dick Kniep <dick.kniep@lindix.nl>
#
# PyHoca GUI is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# PyHoca GUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

modules ={}

try:
    import wxversion
    wxversion.select('2.9')
    #print 'USING wxPython 2.9'

except: pass

try:
    import wxversion
    wxversion.select('2.8')
    #print 'USING wxPython 2.8'

except: pass

import os

# Python X2go
import x2go
import gevent
import gevent.monkey
gevent.monkey.patch_all()

import wx

import time
import sys
import platform
import types
import copy
import threading

# PyHoca-GUI modules
import about
import logon
import taskbar
import menus_taskbar
import profilemanager
import printingprefs
import notify
import basepath
import messages
import splash

wx.SetDefaultPyEncoding("utf-8")
wx.InitAllImageHandlers()

class PyHocaGUI(wx.App, x2go.X2goClient):

    def __init__(self, args, logger, liblogger, 
                 appname='PyHoca-GUI', 
                 vendorname='Open Source Software Foundation',
                 version=None,):
        """\
        STILL UNDOCUMENTED

        """
        self.appname = appname
        self.vendorname = vendorname
        self.version = version

        self.args = args
        if logger is None:
            self._pyhoca_logger = x2go.X2goLogger(tag=self.appname)
        else:
            self._pyhoca_logger = copy.deepcopy(logger)
            self._pyhoca_logger.tag = self.appname

        if liblogger is None:
            self._pyhoca_liblogger = x2go.X2goLogger()
        else:
            self._pyhoca_liblogger = copy.deepcopy(liblogger)

        _x2goclient_kwargs = {
            'use_listsessions_cache': True,
            'auto_update_listsessions_cache': True,
            'auto_update_sessionregistry': True,
            'auto_register_sessions': True,
            'logger': self._pyhoca_liblogger,
        }
        if self.args.backend_controlsession is not None:
            _x2goclient_kwargs['control_backend'] = self.args.backend_controlsession
        if self.args.backend_terminalsession is not None:
            _x2goclient_kwargs['terminal_backend'] = self.args.backend_terminalsession
        if self.args.backend_serversessioninfo is not None:
            _x2goclient_kwargs['info_backend'] = self.args.backend_serversessioninfo
        if self.args.backend_serversessionlist is not None:
            _x2goclient_kwargs['list_backend'] = self.args.backend_serversessionlist
        if self.args.backend_proxy is not None:
            _x2goclient_kwargs['proxy_backend'] = self.args.backend_proxy
        if self.args.backend_sessionprofiles is not None:
            _x2goclient_kwargs['profiles_backend'] = self.args.backend_sessionprofiles
        if self.args.backend_clientsettings is not None:
            _x2goclient_kwargs['settings_backend'] = self.args.backend_clientsettings
        if self.args.backend_clientprinting is not None:
            _x2goclient_kwargs['printing_backend'] = self.args.backend_clientprinting

        if self.args.client_rootdir is not None:
            _x2goclient_kwargs['client_rootdir'] = self.args.client_rootdir
        if self.args.sessions_rootdir is not None:
            _x2goclient_kwargs['sessions_rootdir'] = self.args.sessions_rootdir
        if self.args.ssh_rootdir is not None:
            _x2goclient_kwargs['ssh_rootdir'] = self.args.ssh_rootdir

        if x2go.X2GOCLIENT_OS == 'Windows':
            _x2goclient_kwargs['start_xserver'] = self.args.start_xserver
            _x2goclient_kwargs['start_pulseaudio'] = self.args.start_pulseaudio

        if x2go.X2GOCLIENT_OS == 'Windows' and self.args.start_pulseaudio and os.environ.has_key('PYHOCAGUI_DEVELOPMENT') and os.environ['PYHOCAGUI_DEVELOPMENT'] == '1':
            _x2goclient_kwargs['pulseaudio_installdir'] = os.path.dirname(basepath.pulseaudio_binary)

        try:
            if self.args.logon_window_position:
                self.logon_window_position_x = int(self.args.logon_window_position.split('x')[0])
                self.logon_window_position_y = int(self.args.logon_window_position.split('x')[1])
            else:
                raise
        except:
            self.logon_window_position_x = self.logon_window_position_y = None

        x2go.X2goClient.__init__(self, **_x2goclient_kwargs)
        # We register one session per available session profile.
        # These registered sessions will be used to access the profile's 
        # ,,control'' sessions at first. Later, any session can be used
        # to access the profile's control session.
        self.register_all_session_profiles(return_objects=True)

        wx.App.__init__(self)
        self.Bind(wx.EVT_IDLE, self.OnIdle)


    def OnInit(self):
        """\
        STILL UNDOCUMENTED

        """
        wx.BeginBusyCursor()

        self.SetAppName(self.appname)
        self.SetVendorName(self.vendorname)
        self.startGUI()

        if not self.args.disable_splash:
            splash.PyHocaGUI_SplashScreen()

        wx.EndBusyCursor()
        return True

    def OnIdle(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        try:
            gevent.sleep(.04)
        except KeyboardInterrupt:
            self._pyhoca_logger('Received Ctrl-C keyboard interrupt... Wait till %s has exited cleanly.' % self.appname, loglevel=x2go.loglevel_NOTICE)
            self.WakeUpIdle()
            self.ExitMainLoop()
        except SystemExit:
            self._pyhoca_logger('Received SIGTERM signal... Wait till %s has exited cleanly.' % self.appname, loglevel=x2go.loglevel_NOTICE)
            self.WakeUpIdle()
            self.ExitMainLoop()
        evt.RequestMore()
        return True

    def startGUI(self):
        """\
        STILL UNDOCUMENTED

        """
        # cmd line options
        self.start_on_connect = self.args.start_on_connect
        self.resume_newest_on_connect = self.args.resume_newest_on_connect
        self.resume_oldest_on_connect = self.args.resume_oldest_on_connect
        self.resume_all_on_connect = self.args.resume_all_on_connect
        self.exit_on_disconnect = self.args.exit_on_disconnect
        self.disconnect_on_suspend = self.args.disconnect_on_suspend
        self.disconnect_on_terminate = self.args.disconnect_on_terminate
        self.show_profile_metatypes = self.args.show_profile_metatypes
        self.restricted_trayicon = self.args.restricted_trayicon

        self._pyhoca_logger('PyHoca GUI is starting up', loglevel=x2go.log.loglevel_INFO, )
        self._pyhoca_logger('registering PyHocaGUI control sessions', loglevel=x2go.log.loglevel_INFO, )

        self.about = about.PyHocaGUI_AboutFrame(self)
        self.about.Show(False)

        self.taskbar = taskbar.PyHocaGUI_TaskBarIcon(self.about)
        self.taskbar.Bind(wx.EVT_TASKBAR_LEFT_DCLICK, lambda _Show: self.about.Show(True))

        if not self.restricted_trayicon:
            self.taskbar.Bind(wx.EVT_TASKBAR_LEFT_DOWN, self.taskbar.CreateSessionManagerPopupMenu)

        if x2go.X2GOCLIENT_OS in ('Linux', 'Mac'):
            self.notifier = notify.libnotify_NotifierPopup(self)
        if x2go.X2GOCLIENT_OS in ('Windows'):
            self.notifier = notify.notificationmessage_NotifierPopup(self.about)

        self._sub_windows = []
        self._logon_windows = {}
        self._hide_notifications_map = {}
        self._eventid_profilenames_map = {}
        self._eventid_sessionnames_map = {}
        self._eventid_shared_folders_map = {}
        self._eventid_unshared_folders_map = {}
        self._temp_disabled_profile_names = []
        self._temp_disabled_session_names = {}

        self.profilemanager_disabled = self.args.disable_profilemanager
        self.printingprefs_disabled = self.args.disable_printingprefs

        ###
        ### disable functionality for release versions
        ###
        if os.environ.has_key('PYHOCAGUI_DEVELOPMENT') and os.environ['PYHOCAGUI_DEVELOPMENT'] == '1':
            self.options_disabled = self.args.disable_options
        else:
            self._pyhoca_logger('the current release of %s does not support client configuration' % self.appname, loglevel=x2go.log.loglevel_WARN)
            self.options_disabled = True

        if self.args.auto_connect:
            gevent.spawn(self._auto_connect)

        if self.args.session_profile:
            for profile_name in self.args.session_profile.split(','):
                if not self._X2goClient__client_registered_sessions_of_profile_name(profile_name):
                    continue
                _dummy_id = wx.NewId()
                self._eventid_profilenames_map[_dummy_id] = profile_name
                evt = wx.CommandEvent()
                evt.SetId(_dummy_id)
                self.start_on_connect = True
                self.resume_newest_on_connect = True
                self._pyhoca_logger('opening default session profile %s' % profile_name, loglevel=x2go.log.loglevel_NOTICE)
                self.OnSessionAuthenticate(evt)

    def _auto_connect(self):

        # wait for splash to appear
        if not self.args.disable_splash:
            gevent.sleep(1)
        for session_uuid in self._X2goClient__client_registered_sessions(return_objects=False):
            if self._X2goClient__session_can_auto_connect(session_uuid):
                self._pyhoca_logger('auto-connecting to %s session profile' % self.get_session_profile_name(session_uuid), loglevel=x2go.log.loglevel_NOTICE)

                # create a dummy event and call OnAuthenticate
                _dummy_id = wx.NewId()
                self._eventid_profilenames_map[_dummy_id] = self.get_session_profile_name(session_uuid)
                evt = wx.CommandEvent()
                evt.SetId(_dummy_id)
                self.OnSessionAuthenticate(evt)

    # wx.App's OnExit method
    def OnExit(self):
        """\
        STILL UNDOCUMENTED

        """
        # close open password dialogs (or other remaining windows)
        for _win in self._sub_windows:
            _win.Close()
            _win.Destroy()
        for session_obj in self._X2goClient__client_running_sessions(return_objects=True):
            session_obj.suspend()
        x2go.x2go_cleanup()
        self.about.Close()
        self.about.Destroy()
        self.taskbar.Close()
        self.Destroy()

    # the taskbar's OnExit method...
    def OnTaskbarExit(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self._pyhoca_logger('exit application', loglevel=x2go.log.loglevel_INFO, )
        self.Exit()

    def _start_on_connect(self, evt, session_uuid):
        if not self._X2goClient__list_sessions(session_uuid):
            self._eventid_profilenames_map[evt.GetId()] = self._X2goClient__get_session_profile_name(session_uuid)
            self.OnSessionStart(evt)

    def _resume_newest_on_connect(self, evt, session_uuid):
        session_infos = self._X2goClient__list_sessions(session_uuid)
        if session_infos: 
            newest_session_name = x2go.utils.session_names_by_timestamp(session_infos)[-1]
            self._resume_on_connect(evt, session_uuid, newest_session_name)
            return True
        return False

    def _resume_oldest_on_connect(self, evt, session_uuid):
        session_infos = self._X2goClient__list_sessions(session_uuid)
        if session_infos:
            newest_session_name = x2go.utils.session_names_by_timestamp(session_infos)[0]
            self._resume_on_connect(evt, session_uuid, newest_session_name)
            return True
        return False

    def _resume_all_on_connect(self, evt, session_uuid):
        session_infos = self._X2goClient__list_sessions(session_uuid)
        if session_infos:
            for session_name in session_infos.keys():
                self._resume_on_connect(evt, session_uuid, session_name)
            return True
        return False

    def _resume_on_connect(self, evt, session_uuid, session_name):
        self._eventid_sessionnames_map[evt.GetId()] = session_name
        self._eventid_profilenames_map[evt.GetId()] = self._X2goClient__get_session_profile_name(session_uuid)
        self.OnSessionResume(evt)

    def _post_authenticate(self, evt, session_uuid):

        try:
            _resumed = False
            _dummy = self.list_sessions(session_uuid, refresh_cache=True)
            if self.resume_newest_on_connect:
                _resumed = self._resume_newest_on_connect(evt, session_uuid)
            elif self.resume_oldest_on_connect:
                _resumed = self._resume_oldest_on_connect(evt, session_uuid)
            elif self.resume_all_on_connect:
                _resumed = self._resume_all_on_connect(evt, session_uuid)
            if not _resumed and self.start_on_connect:
                self._start_on_connect(evt, session_uuid)
        except x2go.X2goSessionRegistryException:
            # there might have been a disconnect event inbetween...
            pass

    def _do_authenticate(self, evt, session_uuid):
        connect_failed = False
        profile_name = self.current_profile_name
        try:
            _can_session_auto_connect = self._X2goClient__session_can_auto_connect(session_uuid)
            _can_sshproxy_auto_connect = self._X2goClient__session_can_sshproxy_auto_connect(session_uuid)
            _session_uses_sshproxy = self._X2goClient__session_uses_sshproxy(session_uuid)
            if _can_session_auto_connect:
                self._X2goClient__connect_session(session_uuid)
                if not self._X2goClient__server_valid_x2gouser(session_uuid):
                    self.notifier.send(_(u'%s - connect failure') % profile_name, _(u'User is not allowed to start X2go sessions!'), icon='session_warning', timeout=10000)
                    self.OnServerDisconnect(evt)
                    try:
                        self._temp_disabled_profile_names.remove(profile_name)
                    except ValueError:
                        pass
                else:
                    self.notifier.send(_(u'%s - connect') % profile_name, _(u'Public SSH key authentication has been successful.'), icon='auth_success', timeout=4000)
                    _dummy = self._X2goClient__list_sessions(session_uuid, refresh_cache=True)
                    self._post_authenticate(evt, session_uuid)
                    try:
                        self._temp_disabled_profile_names.remove(profile_name)
                    except ValueError:
                        pass
            else:
                _logon_window = logon.PyHocaGUI_DialogBoxPassword(self, profile_name, caller=self, sshproxy_auth=(not _can_sshproxy_auto_connect) and _session_uses_sshproxy)
                self._logon_windows[profile_name] = _logon_window
        except x2go.AuthenticationException:
            self._pyhoca_logger('public SSH key authentication to server failed, trying next auth-mechanism', loglevel=x2go.log.loglevel_INFO, )
            _logon_window = logon.PyHocaGUI_DialogBoxPassword(self, profile_name, caller=self, )
            self._logon_windows[profile_name] = _logon_window
        except x2go.X2goSSHProxyAuthenticationException:
            self._pyhoca_logger('public SSH key authentication for SSH proxy failed, trying next auth-mechanism', loglevel=x2go.log.loglevel_INFO, )
            _logon_window = logon.PyHocaGUI_DialogBoxPassword(self, profile_name, caller=self, sshproxy_auth=True )
            self._logon_windows[profile_name] = _logon_window
        except x2go.SSHException, e:
            self.notifier.send(_(u'%s - connect error') % profile_name, u'%s!' % str(e), icon='auth_error', timeout=4000)
            try:
                self._temp_disabled_profile_names.remove(profile_name)
            except ValueError:
                pass
            connect_failed = True
        except x2go.X2goHostKeyException, e:
            self.notifier.send(_(u'%s - host key error') % profile_name, _(u'The remote server\'s host key is invalid or has not been accepted by the user') + '!', icon='auth_error', timeout=4000)
            try:
                self._temp_disabled_profile_names.remove(profile_name)
            except ValueError:
                pass
            connect_failed = True
        except x2go.X2goSSHProxyHostKeyException, e:
            self.notifier.send(_(u'%s - host key error') % profile_name, _(u'The SSH proxy\'s host key is invalid or has not been accepted by the user') + '!', icon='auth_error', timeout=4000)
            try:
                self._temp_disabled_profile_names.remove(profile_name)
            except ValueError:
                pass
            connect_failed = True
        except gevent.dns.DNSError, e:
            self.notifier.send(_(u'%s - connect error') % profile_name, e.strerror, icon='auth_error', timeout=4000)
            try:
                self._temp_disabled_profile_names.remove(profile_name)
            except ValueError:
                pass
            connect_failed = True
        except gevent.socket.error, e:
            self.notifier.send(_(u'%s - connect error') % profile_name, e.strerror, icon='auth_error', timeout=4000)
            try:
                self._temp_disabled_profile_names.remove(profile_name)
            except ValueError:
                pass
            connect_failed = True
        except EOFError, e:
            self.notifier.send(_(u'%s - connect error') % profile_name, _(u'Authentication protocol communication incomplete! Try again...'), icon='auth_error', timeout=4000)
            try:
                self._temp_disabled_profile_names.remove(profile_name)
            except ValueError:
                pass
            connect_failed = True
        except x2go.X2goSSHProxyException, e:
            self.notifier.send(_(u'%s - auth key error') % profile_name, u'%s' % str(e), icon='auth_error', timeout=4000)
            try:
                self._temp_disabled_profile_names.remove(profile_name)
            except ValueError:
                pass
            connect_failed = True
        except x2go.X2goRemoteHomeException, e:
            self._PyHocaGUI.notifier.send(_(u'%s - missing home directory') % self.current_profile_name, _("The remote user's home directory does not exist."), icon='auth_error', timeout=4000)
            try:
                self._temp_disabled_profile_names.remove(profile_name)
            except ValueError:
                pass
            connect_failed = True
        except x2go.X2goSessionException, e:
            self.notifier.send(_(u'%s - auth error') % profile_name, u'%s' % str(e), icon='auth_error', timeout=4000)
            try:
                self._temp_disabled_profile_names.remove(profile_name)
            except ValueError:
                pass
            connect_failed = True
        except:
            self.notifier.send('%s - connect error' % profile_name, _(u'An unknown error occurred during authentication!'), icon='auth_error', timeout=4000)
            try:
                self._temp_disabled_profile_names.remove(profile_name)
            except ValueError:
                pass
            connect_failed = True
            if os.environ.has_key('PYHOCAGUI_DEVELOPMENT') and os.environ['PYHOCAGUI_DEVELOPMENT'] == '1': 
                raise


        if connect_failed and self.exit_on_disconnect:
            self.WakeUpIdle()
            self.ExitMainLoop()

        self.taskbar.SetIconIdle()

    def OnSessionAuthenticate(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        self.taskbar.SetIconConnecting(self.current_profile_name)
        session_uuid = self._X2goClient__register_session(profile_name=self.current_profile_name)
        self._temp_disabled_profile_names.append(self.current_profile_name)
        gevent.spawn(self._do_authenticate, evt, session_uuid)

    def OnSessionStart(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        _query_session = self._X2goClient__client_registered_sessions_of_profile_name(self.current_profile_name)[0]
        session_uuid = self._X2goClient__register_session(profile_name=self.current_profile_name)
        if self._X2goClient__server_is_alive(session_uuid):
            gevent.spawn(self._X2goClient__start_session, session_uuid)
            _dummy = self._X2goClient__list_sessions(session_uuid, refresh_cache=True)

    def _disable_session_name(self, profile_name, session_name):
        if profile_name not in self._temp_disabled_session_names.keys():
            self._temp_disabled_session_names[profile_name] = []
        self._temp_disabled_session_names[profile_name].append(session_name)

    def _enable_session_name(self, profile_name, session_name):
        try:
            self._temp_disabled_session_names[profile_name].remove(session_name)
        except (KeyError, ValueError):
            pass

    def OnSessionResume(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        session_uuid = self._X2goClient__client_registered_sessions_of_profile_name(self.current_profile_name)[0]
        session_name = self._eventid_sessionnames_map[evt.GetId()]
        self._disable_session_name(self.current_profile_name, session_name)
        if self._X2goClient__server_is_alive(session_uuid):
            gevent.spawn(self._X2goClient__resume_session, session_name=session_name)
            _dummy = self._X2goClient__list_sessions(session_uuid, refresh_cache=True)
        self._eventid_sessionnames_map = {}

    def OnSessionSuspend(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        session_uuid = self._X2goClient__client_registered_sessions_of_profile_name(self.current_profile_name)[0]
        session_name = self._eventid_sessionnames_map[evt.GetId()]
        self._disable_session_name(self.current_profile_name, session_name)
        gevent.spawn(self._X2goClient__suspend_session, session_uuid, session_name=session_name)
        self._eventid_sessionnames_map = {}
        if self.disconnect_on_suspend:
            self.OnServerDisconnect(evt)

    def OnSessionTerminate(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        session_uuid = self._X2goClient__client_registered_sessions_of_profile_name(self.current_profile_name)[0]
        session_name = self._eventid_sessionnames_map[evt.GetId()]
        self._disable_session_name(self.current_profile_name, session_name)
        gevent.spawn(self._X2goClient__terminate_session, session_uuid, session_name=session_name)
        self._eventid_sessionnames_map = {}
        if self.disconnect_on_terminate:
            self.OnServerDisconnect(evt)

    def OnCleanSessions(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        session_uuid = self._X2goClient__client_registered_sessions_of_profile_name(self.current_profile_name)[0]
        session_names = self._X2goClient__server_running_sessions(session_uuid) + self._X2goClient__server_suspended_sessions(session_uuid)
        if self._X2goClient__server_is_alive(session_uuid):
            if session_names:
                _notify_text = _(u'Cleaning X2go sessions...')
                self._hide_notifications_map[self.current_profile_name] = []
                for session_name in session_names:
                    _notify_text += '\n%s' % session_name
                    self._hide_notifications_map[self.current_profile_name].append(session_name)
                    self._disable_session_name(self.current_profile_name, session_name)
                self.notifier.send(self.current_profile_name, _notify_text, icon='session_cleanall', timeout=10000)
            gevent.spawn(self._X2goClient__clean_sessions, session_uuid)

    def OnServerDisconnect(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        session_uuid = self._X2goClient__client_registered_sessions_of_profile_name(self.current_profile_name)[0]
        # disconnect all profile sessions
        if self._X2goClient__server_is_alive(session_uuid):
            self._X2goClient__disconnect_profile(self.current_profile_name)
            self.notifier.send(_(u'%s - disconnect') % self.current_profile_name, _(u'X2go Profile is now disconnected.'), icon='auth_disconnect', timeout=4000)
        if self.exit_on_disconnect:
            self._pyhoca_logger('Exiting %s because %s got disconnected.' % (self.appname, self.current_profile_name), loglevel=x2go.loglevel_NOTICE)
            self.WakeUpIdle()
            self.ExitMainLoop()
        try:
            del self._temp_disabled_session_names[self.current_profile_name]
        except KeyError:
            pass

    def OnProfileAdd(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self._pyhoca_logger('adding new X2go session profile', loglevel=x2go.log.loglevel_INFO, )
        profilemanager.PyHocaGUI_ProfileManager(self, 'ADD', profile_name=_(u'New Session Profile'))

    def OnProfileEdit(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        self._pyhoca_logger('editing session profile %s' % self.current_profile_name, loglevel=x2go.log.loglevel_INFO, )
        profilemanager.PyHocaGUI_ProfileManager(self, 'EDIT', profile_name=self.current_profile_name)

    def OnProfileEditWhileConnected(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        self._pyhoca_logger('editing session profile %s' % self.current_profile_name, loglevel=x2go.log.loglevel_INFO, )
        profilemanager.PyHocaGUI_ProfileManager(self, 'EDIT_CONNECTED', profile_name=self.current_profile_name)

    def OnProfileDelete(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]

        m = messages.PyHoca_MessageWindow_NoYes(self, shortmsg='REALLY_DELETE_PROFILE', title=_(u'Really Delete Session Profile ,,%s\'\'?') % self.current_profile_name, icon='question', profile_name=self.current_profile_name)
        m.ShowModal()
        if m.Yes():
            self._pyhoca_logger('deleting session profile %s' % self.current_profile_name, loglevel=x2go.log.loglevel_INFO, )
            try:
                self.session_profiles.delete_profile(self.current_profile_name)
                self.notifier.send(title=_(u'%s - profile deleted') % self.current_profile_name, text=_(u'The session profile has been deleted.'), icon='profile_delete')
            except:
                pass

    def OnShareCustomLocalFolder(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        self._pyhoca_logger('Evoking file dialog for ,,Share Local Folder\'\' menu item action', loglevel=x2go.log.loglevel_NOTICE, )
        shared_folder = os.path.expanduser('~')
        if not os.path.exists(shared_folder):
            shared_folder = os.getcwd()
        dlg = wx.DirDialog(
            self.about, message=_(u"%s - share local folder with sessions of this profile") % self.current_profile_name, style=1, defaultPath=shared_folder)
        # Show the dialog and retrieve the user response. If it is the OK response,
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            # This returns a Python list of files that were selected.
            self._X2goClient__share_local_folder_with_session(profile_name=self.current_profile_name, folder_name=str(dlg.GetPath()))

    def OnUnshareAllLocalFolders(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        self._X2goClient__unshare_all_local_folders_from_profile(profile_name=self.current_profile_name)

    def OnShareLocalFolder(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        _share_folder = self._eventid_unshared_folders_map[evt.GetId()]
        self._X2goClient__share_local_folder(profile_name=self.current_profile_name, local_path=_share_folder)

    def OnUnshareLocalFolder(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.current_profile_name = self._eventid_profilenames_map[evt.GetId()]
        _unshare_folder = self._eventid_shared_folders_map[evt.GetId()]
        self._X2goClient__unshare_local_folder_from_profile(profile_name=self.current_profile_name, local_path=_unshare_folder)

    def OnListSessions(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self._pyhoca_logger('The ,,List Sessions\'\' information window is not implemented yet', loglevel=x2go.log.loglevel_WARN, )

    def OnAbout(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self._pyhoca_logger('Showing the ,,About...\'\' window', loglevel=x2go.log.loglevel_INFO, )
        self.about.Show(True)

    def OnOptions(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self._pyhoca_logger('The ,,Options\'\' configuration window is not implemented yet', loglevel=x2go.log.loglevel_WARN, )

    def OnPrintingPreferences(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self._pyhoca_logger('opening the printing preferences window', loglevel=x2go.log.loglevel_INFO, )
        _pp_dialog = printingprefs.PyHocaGUI_PrintingPreferences(self)

    def OnClose(self, evt):
        """\
        STILL UNDOCUMENTED

        """
        self.OnExit(evt)

    ##
    ## Python X2go (X2goClient) interactive HOOK's...
    ##
    def HOOK_check_host_dialog(self, profile_name='UNKNOWN', host='UNKNOWN', port=22, fingerprint='no fingerprint', fingerprint_type='RSA', ):
        """\
        STILL UNDOCUMENTED

        """
        _message = _(u'The authenticity of host [%s]:%s can\'t be established.\n%s key fingerprint is ,,%s\'\'.\n\nAre you sure you want to continue connecting?') % (host, port, fingerprint_type, fingerprint)

        if self._logon_windows.has_key(profile_name):
            _parent = self._logon_windows[profile_name]
        else:
            # use a dummy parent...
            _parent = None

        m = messages.PyHoca_MessageWindow_NoYes(self, parent=_parent, custom_message=_message, title=_(u'%s: Confirm Host Authorization') % profile_name, icon='profile_warning')

        if _parent:
            m.ShowModal()
        else:
            m.Show()
            while m in self._sub_windows:
                gevent.sleep(.04)

        retval = m.Yes()
        m.Destroy()
        return retval

    ##
    ## Python X2go (X2goClient) notification HOOK's...
    ##

    # this hook gets called from Python X2go classes if profile_name's control session has died...
    def HOOK_on_control_session_death(self, profile_name):
        self.notifier.send(_(u'%s - channel error') % profile_name, _(u'Lost connection to server %s unexpectedly! Try to re-authenticate to the server...') % profile_name, icon='session_warning', timeout=10000)
        try:
            del self._temp_disabled_session_names[profile_name]
        except KeyError:
            pass
        if self.exit_on_disconnect:
            self.WakeUpIdle()
            self.ExitMainLoop()

    def HOOK_session_startup_failed(self, profile_name='UNKNOWN'):
        self.notifier.send(_(u'%s - session failure') % profile_name, _('The session startup failed.'), icon='session_error', timeout=10000)
        if self.exit_on_disconnect:
            self.WakeUpIdle()
            self.ExitMainLoop()

    def HOOK_no_such_command(self, cmd, profile_name='UNKNOWN', session_name='UNKNOWN'):
        if session_name == 'UNKNOWN':
            self.notifier.send(_(u'%s - session failure') % profile_name, _('The command ,,%s\'\' is not available on X2go server.') % cmd, icon='session_error', timeout=10000)
        else:
            self.notifier.send(_(u'%s - session failure') % profile_name, _('The command ,,%s\'\' is not available on X2go server\n%s.') % (cmd, session_name), icon='session_error', timeout=10000)
        if self.exit_on_disconnect:
            self.WakeUpIdle()
            self.ExitMainLoop()

    def HOOK_rforward_request_denied(self, profile_name='UNKNOWN', session_name='UNKNOWN', server_port=0):
        self.notifier.send(_(u'%s - session warning') % profile_name, _(u'Reverse TCP port forwarding request for session %s to server port %s has been denied.') % (session_name, server_port), icon='session_warning', timeout=10000)
        if self.exit_on_disconnect:
            self.WakeUpIdle()
            self.ExitMainLoop()

    def HOOK_forwarding_tunnel_setup_failed(self, profile_name='UNKNOWN', session_name='UNKNOWN', chain_host='UNKNOWN', chain_port=0):
        self.notifier.send(_(u'%s - session failure') % profile_name, _(u'Forwarding tunnel request to [%s]:%s for session %s was denied by remote X2go/SSH server. Session startup failed.') % (chain_host, chain_port, session_name), icon='session_error', timeout=10000)
        try:
            self._temp_disabled_session_names[profile_name].remove(session_name)
        except KeyError:
            pass
        except ValueError:
            pass
        if self.exit_on_disconnect:
            self.WakeUpIdle()
            self.ExitMainLoop()

    def HOOK_pulseaudio_not_supported_in_RDPsession(self):
        self.notifier.send(_(u'%s - audio warning') % self.appname, _(u'The X2go PulseAudio system is not available within Remote Desktop sessions.'), icon='audio_error', timeout=10000)

    def HOOK_pulseaudio_server_startup_failed(self):
        self.notifier.send(_(u'%s - audio error') % self.appname, _(u'The X2go PulseAudio system could not be started.'), icon='audio_error', timeout=10000)

    def HOOK_pulseaudio_server_died(self):
        self.notifier.send(_(u'%s - audio error') % self.appname, _(u'The X2go PulseAudio system has died unexpectedly.'), icon='audio_error', timeout=10000)

    def HOOK_on_sound_tunnel_failed(self, profile_name='UNKNOWN', session_name='UNKNOWN'):
        self.notifier.send(_(u'%s - audio problem') % profile_name, _(u'The audio connection could not be set up for this session.\n%s') % session_name, icon='session_warning', timeout=5000)

    # this hook gets called from Python X2go classes if a print job is coming in and the print action is ,,DIALOG''...
    def HOOK_open_print_dialog(self, profile_name='UNKNOWN', session_name='UNKNOWN'):
        _print_action = None
        _pp_dialog = printingprefs.PyHocaGUI_PrintingPreferences(self, mode='print', profile_name=profile_name, session_name=session_name)
        while _pp_dialog in self._sub_windows:
            _print_action = _pp_dialog.get_print_action()
            gevent.sleep(.2)

        return _print_action

    def HOOK_printaction_error(self, filename, profile_name='UNKNOWN', session_name='UNKNOWN', err_msg='GENERIC_ERROR', printer=None):
        if printer:
            self.notifier.send(_(u'%s - print error') % profile_name, _(u'%s\n...caused on printer %s by session\n%s')  % (err_msg, printer, session_name), icon='session_error', timeout=5000)
        else:
            self.notifier.send(_(u'%s - print error') % profile_name, _(u'%s\n...caused by session\n%s')  % (err_msg, session_name), icon='session_error', timeout=5000)

    def HOOK_on_session_has_started_by_me(self, session_uuid='UNKNOWN', profile_name='UNKNOWN', session_name='UNKNOWN'):
        self._enable_session_name(profile_name, session_name)
        self.notifier.send(_(u'%s - start') % profile_name, _(u'New X2go session starting up...\n%s') % session_name, icon='session_start', timeout=5000)

    def HOOK_on_session_has_started_by_other(self, session_uuid='UNKNOWN', profile_name='UNKNOWN', session_name='UNKNOWN'):
        self._enable_session_name(profile_name, session_name)
        self.notifier.send(_(u'%s - start') % profile_name, _(u'Another client started X2go session\n%s') % session_name, icon='session_start', timeout=5000)

    def HOOK_on_session_has_resumed_by_me(self, session_uuid='UNKNOWN', profile_name='UNKNOWN', session_name='UNKNOWN'):
        self._enable_session_name(profile_name, session_name)
        self.notifier.send(_(u'%s - resume') % profile_name, _(u'Resuming X2go session...\n%s') % session_name, icon='session_resume', timeout=5000)

    def HOOK_on_session_has_resumed_by_other(self, session_uuid='UNKNOWN', profile_name='UNKNOWN', session_name='UNKNOWN'):
        self._enable_session_name(profile_name, session_name)
        self.notifier.send(_(u'%s - resume') % profile_name, _(u'Another client resumed X2go session\n%s') % session_name, icon='session_resume', timeout=5000)

    def HOOK_on_found_session_running_after_connect(self, session_uuid='UNKNOWN', profile_name='UNKNOWN', session_name='UNKNOWN'):
        self._enable_session_name(profile_name, session_name)
        gevent.spawn_later(5, self.notifier.send, _(u'%s - running') % profile_name, _(u'Found already running session\n%s') %  session_name, icon='session_resume', timeout=5000)

    def HOOK_on_session_has_been_suspended(self, session_uuid='UNKNOWN', profile_name='UNKNOWN', session_name='UNKNOWN'):
        self._enable_session_name(profile_name, session_name)
        self.notifier.send(_(u'%s - suspend') % profile_name, _(u'X2go Session has been suspended\n%s') % session_name, icon='session_suspend', timeout=5000)
        if self.disconnect_on_suspend:
            _dummy_id = wx.NewId()
            self._eventid_profilenames_map[_dummy_id] = profile_name
            evt = wx.IdleEvent()
            evt.SetId(_dummy_id)
            self.OnServerDisconnect(dummy_evt)

    def HOOK_on_session_has_terminated(self, session_uuid='UNKNOWN', profile_name='UNKNOWN', session_name='UNKNOWN'):
        self._enable_session_name(profile_name, session_name)
        # avoid notification if X2goClient.clean_sessions has been used to terminate sessions
        if self._hide_notifications_map.has_key(profile_name) and session_name in self._hide_notifications_map[profile_name]:
            self._hide_notifications_map[profile_name].remove(session_name)
            if not self._hide_notifications_map[profile_name]:
                del self._hide_notifications_map[profile_name]
        else:
            self.notifier.send(_(u'%s - terminate') % profile_name, _(u'X2go Session has terminated\n%s') % session_name, icon='session_terminate', timeout=5000)
        if self.disconnect_on_terminate:
            _dummy_id = wx.NewId()
            self._eventid_profilenames_map[_dummy_id] = profile_name
            evt = wx.IdleEvent()
            evt.SetId(_dummy_id)
            self.OnServerDisconnect(evt)
