# -*- coding: utf-8 -*-

# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
# Copyright (C) 2010-2011 by Dick Kniep <dick.kniep@lindix.nl>
#
# PyHoca GUI is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# PyHoca GUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import os
import wx
from x2go import X2GOCLIENT_OS
from x2go import log
if X2GOCLIENT_OS in ('Linux', 'Mac'):
    import pynotify
import exceptions
import basepath

import x2go.utils as utils

_icons_location = basepath.icons_basepath

class NotSupportedException(exceptions.StandardError): pass
class PyHocaNotificationException(exceptions.StandardError): pass

class libnotify_NotifierPopup(object):

    title = {}
    text = {}
    icon = {}
    timeout = {}

    def __init__(self, _PyHocaGUI):
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

        if not pynotify.init("PyHocaGUI"):
            raise NotSupportedException

    def prepare(self, context, title=None, text=None, icon=None, timeout=None):
        if title is not None:
            self.title[context] = title
        if text is not None:
            self.text[context] = text
        if icon is not None:
            self.icon[context] = icon
        if timeout is not None:
            self.timeout[context] = timeout

    def send(self, title=None, text=None, context=None, icon=None, timeout=8000):
        if context is not None:
            try:
                title = self.title[context]
                del self.title[context]
            except KeyError:
                pass
            try:
                text = self.text[context]
                del self.text[context]
            except KeyError:
                pass
            try:
                icon = self.icon[context]
                del self.icon[context]
            except KeyError:
                pass
            try:
                timeout = self.timeout[context]
                del self.timeout[context]
            except KeyError:
                pass

        icon = 'file://%s/PyHoca/32x32/%s.png' % (_icons_location, icon)

        n = pynotify.Notification(title, text, icon)
        n.set_urgency(pynotify.URGENCY_NORMAL)
        n.set_timeout(timeout)

        self._pyhoca_logger('[%s] %s' % (title.encode(utils.get_encoding()), text.encode(utils.get_encoding())), loglevel=log.loglevel_NOTICE)
        if not n.show():
            raise PyHocaNotificationException('could not notify user')


    def Close(self):
        pass

    def Destroy(self):
        pass


class notificationmessage_NotifierPopup(object):

    title = {}
    text = {}
    icon = {}

    def __init__(self, _about):
        self._PyHocaGUI = _about._PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

    def prepare(self, context, title=None, text=None, icon=None):
        if title is not None:
            self.title[context] = title
        if text is not None:
            self.text[context] = text
        if icon is not None:
            self.icon[context] = icon

    def send(self, title=None, text=None, context=None, icon=None, timeout=8000):
        if context is not None:
            try:
                title = self.title[context]
                del self.title[context]
            except KeyError:
                pass
            try:
                text = self.text[context]
                del self.text[context]
            except KeyError:
                pass
            try:
                icon = self.icon[context]
                del self.icon[context]
            except KeyError:
                pass

        _icon_map_wx = {
            'audio_error': wx.ICON_ERROR,
            'auth_success': wx.ICON_INFORMATION,
            'auth_failed': wx.ICON_WARNING,
            'auth_error': wx.ICON_ERROR,
            'auth_disconnect': wx.ICON_INFORMATION,
            'profile_add': wx.ICON_INFORMATION,
            'profile_delete': wx.ICON_INFORMATION,
            'profile_edit': wx.ICON_INFORMATION,
            'profile_save': wx.ICON_INFORMATION,
            'profile_error': wx.ICON_ERROR,
            'session_cleanall': wx.ICON_INFORMATION,
            'session_error': wx.ICON_ERROR,
            'session_pause': wx.ICON_INFORMATION,
            'session_printing': wx.ICON_INFORMATION,
            'session_resume': wx.ICON_INFORMATION,
            'session_start': wx.ICON_INFORMATION,
            'session_terminate': wx.ICON_INFORMATION,
            'session_warning': wx.ICON_WARNING,
        }
        if icon in _icon_map_wx.keys():
           icon = _icon_map_wx[icon]
        else:
           icon = wx.ICON_INFORMATION

        try:
            # you will need wxPython >= 2.9 for this
            _notification_msg = wx.NotificationMessage()
            _notification_msg.SetTitle(title)
            _notification_msg.SetMessage(text)
            _notification_msg.SetParent(self._PyHocaGUI.about)
            _notification_msg.SetFlags(icon)
            _notification_msg.Show(timeout=1)
        except:
            # if we are running wxPython 2.8, we ignore missing 
            # wx.NotificationMessage class
            pass

        # on Windows some error messages are already encoded, some are not, depending from which module they come
        try: _title = title.encode(utils.get_encoding())
        except: _title = title
        try: _text = text.encode(utils.get_encoding())
        except: _text = text

        try: self._pyhoca_logger('['+_title+'] '+_text, loglevel=log.loglevel_NOTICE)
        except UnicodeDecodeError: self._pyhoca_logger('Unicode error occurred while rendering a log message...', loglevel=log.loglevel_WARN)

    def Close(self):
        pass

    def Destroy(self):
        pass
